@@ -11,6 +11,8 @@ import (
 	"sync/atomic"
 	"testing"
 
+	"go.k6.io/k6/lib"
+
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -82,7 +84,7 @@ func NewGlobalTestState(tb testing.TB) *GlobalTestState {
 	})
 
 	outMutex := &sync.Mutex{}
-	defaultFlags := state.GetDefaultFlags(".config")
+	defaultFlags := state.GetDefaultFlags(".config", ".cache")
 	defaultFlags.Address = getFreeBindAddr(tb)
 
 	ts.GlobalState = &state.GlobalState{
@@ -113,6 +115,7 @@ func NewGlobalTestState(tb testing.TB) *GlobalTestState {
 		Logger:         logger,
 		FallbackLogger: testutils.NewLogger(tb).WithField("fallback", true),
 		Usage:          usage.New(),
+		TestStatus:     lib.NewTestStatus(),
 	}
 
 	return ts
