@@ -6,6 +6,8 @@ import (
 	"sync"
 
 	"github.com/grafana/sobek"
+
+	"go.k6.io/k6/js/common"
 	"go.k6.io/k6/js/modules"
 )
 
@@ -185,7 +187,7 @@ func (e *EventLoop) Start(firstCallback func() error) error {
 		// But that seems to be the case in other tools as well so it seems to not be that big of a problem.
 		for promise := range e.pendingPromiseRejections {
 			value := promise.Result()
-			if !sobek.IsNull(value) && !sobek.IsUndefined(value) {
+			if !common.IsNullish(value) {
 				if o := value.ToObject(e.vu.Runtime()); o != nil {
 					if stack := o.Get("stack"); stack != nil {
 						value = stack
