@@ -1,16 +1,20 @@
 package browser
 
 import (
+	"errors"
 	"fmt"
 
 	"github.com/grafana/sobek"
 
 	"go.k6.io/k6/internal/js/modules/k6/browser/common"
 	"go.k6.io/k6/internal/js/modules/k6/browser/k6ext"
+	k6common "go.k6.io/k6/js/common"
 )
 
 // mapLocator API to the JS module.
-func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
+//
+//nolint:gocognit,funlen
+func mapLocator(vu moduleVU, lo *common.Locator) mapping {
 	rt := vu.Runtime()
 	return mapping{
 		"all": func() *sobek.Promise {
@@ -27,6 +31,15 @@ func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
 				return res, nil
 			})
 		},
+		"boundingBox": func(opts sobek.Value) (*sobek.Promise, error) {
+			popts := common.NewFrameBaseOptions(lo.Timeout())
+			if err := popts.Parse(vu.Context(), opts); err != nil {
+				return nil, fmt.Errorf("parsing locator bounding box options: %w", err)
+			}
+			return k6ext.Promise(vu.Context(), func() (any, error) {
+				return lo.BoundingBox(popts) //nolint:wrapcheck
+			}), nil
+		},
 		"clear": func(opts sobek.Value) (*sobek.Promise, error) {
 			copts := common.NewFrameFillOptions(lo.Timeout())
 			if err := copts.Parse(vu.Context(), opts); err != nil {
@@ -46,6 +59,10 @@ func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
 				return nil, lo.Click(popts) //nolint:wrapcheck
 			}), nil
 		},
+		"contentFrame": func() *sobek.Object {
+			ml := mapFrameLocator(vu, lo.ContentFrame())
+			return rt.ToValue(ml).ToObject(rt)
+		},
 		"count": func() *sobek.Promise {
 			return k6ext.Promise(vu.Context(), func() (any, error) {
 				return lo.Count() //nolint:wrapcheck
@@ -106,6 +123,11 @@ func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
 				return nil, lo.Fill(value, opts) //nolint:wrapcheck
 			})
 		},
+		"filter": func(opts sobek.Value) mapping {
+			return mapLocator(vu, lo.Filter(&common.LocatorFilterOptions{
+				LocatorOptions: parseLocatorOptions(rt, opts),
+			}))
+		},
 		"first": func() *sobek.Object {
 			ml := mapLocator(vu, lo.First())
 			return rt.ToValue(ml).ToObject(rt)
@@ -127,6 +149,72 @@ func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
 				return s, nil
 			})
 		},
+		"getByAltText": func(alt sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(alt) {
+				return nil, errors.New("missing required argument 'altText'")
+			}
+			palt, popts := parseGetByBaseOptions(vu.Context(), alt, false, opts)
+
+			ml := mapLocator(vu, lo.GetByAltText(palt, popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByLabel": func(label sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(label) {
+				return nil, errors.New("missing required argument 'label'")
+			}
+			plabel, popts := parseGetByBaseOptions(vu.Context(), label, true, opts)
+
+			ml := mapLocator(vu, lo.GetByLabel(plabel, popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByPlaceholder": func(placeholder sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(placeholder) {
+				return nil, errors.New("missing required argument 'placeholder'")
+			}
+			pplaceholder, popts := parseGetByBaseOptions(vu.Context(), placeholder, false, opts)
+
+			ml := mapLocator(vu, lo.GetByPlaceholder(pplaceholder, popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByRole": func(role sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(role) {
+				return nil, errors.New("missing required argument 'role'")
+			}
+			popts := parseGetByRoleOptions(vu.Context(), opts)
+
+			ml := mapLocator(vu, lo.GetByRole(role.String(), popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByTestId": func(testID sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(testID) {
+				return nil, errors.New("missing required argument 'testId'")
+			}
+			ptestID := parseStringOrRegex(testID, false)
+
+			ml := mapLocator(vu, lo.GetByTestID(ptestID))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByText": func(text sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(text) {
+				return nil, errors.New("missing required argument 'text'")
+			}
+			ptext, popts := parseGetByBaseOptions(vu.Context(), text, true, opts)
+
+			ml := mapLocator(vu, lo.GetByText(ptext, popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"getByTitle": func(title sobek.Value, opts sobek.Value) (*sobek.Object, error) {
+			if k6common.IsNullish(title) {
+				return nil, errors.New("missing required argument 'title'")
+			}
+			ptitle, popts := parseGetByBaseOptions(vu.Context(), title, false, opts)
+
+			ml := mapLocator(vu, lo.GetByTitle(ptitle, popts))
+			return rt.ToValue(ml).ToObject(rt), nil
+		},
+		"locator": func(selector string, opts sobek.Value) mapping {
+			return mapLocator(vu, lo.Locator(selector, parseLocatorOptions(rt, opts)))
+		},
 		"innerHTML": func(opts sobek.Value) *sobek.Promise {
 			return k6ext.Promise(vu.Context(), func() (any, error) {
 				return lo.InnerHTML(opts) //nolint:wrapcheck
@@ -207,3 +295,23 @@ func mapLocator(vu moduleVU, lo *common.Locator) mapping { //nolint:funlen
 		},
 	}
 }
+
+func parseLocatorOptions(rt *sobek.Runtime, opts sobek.Value) *common.LocatorOptions {
+	if k6common.IsNullish(opts) {
+		return nil
+	}
+
+	var popts common.LocatorOptions
+
+	obj := opts.ToObject(rt)
+	for _, k := range obj.Keys() {
+		switch k {
+		case "hasText":
+			popts.HasText = obj.Get(k).String()
+		case "hasNotText":
+			popts.HasNotText = obj.Get(k).String()
+		}
+	}
+
+	return &popts
+}
