@@ -339,7 +339,7 @@ func mapPage(vu moduleVU, p *common.Page) mapping { //nolint:gocognit,cyclop
 		},
 		"keyboard": mapKeyboard(vu, p.GetKeyboard()),
 		"locator": func(selector string, opts sobek.Value) *sobek.Object {
-			ml := mapLocator(vu, p.Locator(selector, opts))
+			ml := mapLocator(vu, p.Locator(selector, parseLocatorOptions(rt, opts)))
 			return rt.ToValue(ml).ToObject(rt)
 		},
 		"mainFrame": func() *sobek.Object {
@@ -580,6 +580,35 @@ func mapPage(vu moduleVU, p *common.Page) mapping { //nolint:gocognit,cyclop
 		"waitForURL": func(url sobek.Value, opts sobek.Value) (*sobek.Promise, error) {
 			return waitForURLBodyImpl(vu, p, url, opts)
 		},
+		"waitForResponse": func(url sobek.Value, opts sobek.Value) (*sobek.Promise, error) {
+			popts, err := parseWaitForResponseOptions(vu.Context(), opts, p.Timeout())
+			if err != nil {
+				return nil, fmt.Errorf("parsing waitForResponse options: %w", err)
+			}
+
+			var val string
+			switch url.ExportType() {
+			case reflect.TypeOf(string("")):
+				val = "'" + url.String() + "'" // Strings require quotes
+			default: // JS Regex, CSS, numbers or booleans
+				val = url.String() // No quotes
+			}
+
+			// Inject JS regex checker for URL pattern matching
+			ctx, stopTaskqueue := context.WithCancel(vu.Context())
+			tq := cancelableTaskQueue(ctx, vu.RegisterCallback)
+
+			jsRegexChecker, err := injectRegexMatcherScript(ctx, vu, tq)
+			if err != nil {
+				stopTaskqueue()
+				return nil, err
+			}
+
+			return k6ext.Promise(ctx, func() (result any, reason error) {
+				defer stopTaskqueue()
+				return p.WaitForResponse(val, popts, jsRegexChecker)
+			}), nil
+		},
 		"workers": func() *sobek.Object {
 			var mws []mapping
 			for _, w := range p.Workers() {
@@ -804,13 +833,15 @@ func parseWaitForFunctionArgs(
 // so that it is easier to copy over updates/fixes from Playwright when we need
 // to.
 func parseStringOrRegex(v sobek.Value, doubleQuote bool) string {
+	const stringType = string("")
+
 	var a string
 	switch v.ExportType() {
-	case reflect.TypeOf(string("")): // text values require quotes
+	case reflect.TypeOf(stringType): // text values require quotes
 		if doubleQuote {
-			a = fmt.Sprintf(`"%s"`, v.String())
+			a = `"` + v.String() + `"`
 		} else {
-			a = fmt.Sprintf("'%s'", v.String())
+			a = `'` + v.String() + `'`
 		}
 	case reflect.TypeOf(map[string]interface{}(nil)): // JS RegExp
 		a = v.String() // No quotes
@@ -822,7 +853,7 @@ func parseStringOrRegex(v sobek.Value, doubleQuote bool) string {
 
 // parseGetByRoleOptions parses the GetByRole options from the Sobek.Value.
 func parseGetByRoleOptions(ctx context.Context, opts sobek.Value) *common.GetByRoleOptions {
-	if !sobekValueExists(opts) {
+	if k6common.IsNullish(opts) {
 		return nil
 	}
 
@@ -876,7 +907,7 @@ func parseGetByBaseOptions(
 ) (string, *common.GetByBaseOptions) {
 	a := parseStringOrRegex(input, doubleQuote)
 
-	if !sobekValueExists(opts) {
+	if k6common.IsNullish(opts) {
 		return a, nil
 	}
 
@@ -956,13 +987,7 @@ func waitForURLBodyImpl(vu moduleVU, target interface {
 		return nil, fmt.Errorf("parsing waitForURL options: %w", err)
 	}
 
-	var val string
-	switch url.ExportType() {
-	case reflect.TypeOf(string("")):
-		val = fmt.Sprintf("'%s'", url.String()) // Strings require quotes
-	default: // JS Regex, CSS, numbers or booleans
-		val = url.String() // No quotes
-	}
+	val := parseStringOrRegex(url, false)
 
 	// Inject JS regex checker for URL pattern matching
 	ctx, stopTaskqueue := context.WithCancel(vu.Context())
