@@ -17,9 +17,10 @@ import (
 //
 // See Issue #100 for more details.
 
-// Locator represent a way to find element(s) on the page at any moment.
+// Locator represents a way to find element(s) on the page at any moment.
 type Locator struct {
 	selector string
+	opts     *LocatorOptions
 
 	frame *Frame
 
@@ -27,16 +28,42 @@ type Locator struct {
 	log *log.Logger
 }
 
+// LocatorOptions allows modifying the [Locator] behavior.
+type LocatorOptions struct {
+	// Matches only elements that contain the specified text.
+	// String or RegExp. Optional.
+	HasText string
+	// Matches only elements that do not contain the specified text.
+	// String or RegExp. Optional.
+	HasNotText string
+}
+
 // NewLocator creates and returns a new locator.
-func NewLocator(ctx context.Context, selector string, f *Frame, l *log.Logger) *Locator {
+func NewLocator(ctx context.Context, opts *LocatorOptions, selector string, f *Frame, l *log.Logger) *Locator {
+	if opts == nil {
+		opts = new(LocatorOptions)
+	}
+	if opts.HasText != "" {
+		selector += " >> internal:has-text=" + opts.HasText
+	}
+	if opts.HasNotText != "" {
+		selector += " >> internal:has-not-text=" + opts.HasNotText
+	}
 	return &Locator{
 		selector: selector,
+		opts:     opts,
 		frame:    f,
 		ctx:      ctx,
 		log:      l,
 	}
 }
 
+// BoundingBox will return the bounding box of the element.
+func (l *Locator) BoundingBox(opts *FrameBaseOptions) (*Rect, error) {
+	opts.Strict = true
+	return l.frame.boundingBox(l.selector, opts)
+}
+
 // Clear will clear the input field.
 // This works with the Fill API and fills the input field with an empty string.
 func (l *Locator) Clear(opts *FrameFillOptions) error {
@@ -78,6 +105,7 @@ func (l *Locator) Click(opts *FrameClickOptions) error {
 // error, or applies slow motion.
 func (l *Locator) click(opts *FrameClickOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.click(l.selector, opts)
 }
 
@@ -97,6 +125,13 @@ func (l *Locator) All() ([]*Locator, error) {
 	return locators, nil
 }
 
+// ContentFrame creates and returns a new FrameLocator, which is useful when
+// needing to interact with elements in an iframe and the current locator already
+// points to the iframe.
+func (l *Locator) ContentFrame() *FrameLocator {
+	return NewFrameLocator(l.ctx, l.selector, l.frame, l.log)
+}
+
 // Count APIs do not wait for the element to be present. It also does not set
 // strict to true, allowing it to return the total number of elements matching
 // the selector.
@@ -125,6 +160,7 @@ func (l *Locator) Dblclick(opts sobek.Value) error {
 // error, or applies slow motion.
 func (l *Locator) dblclick(opts *FrameDblclickOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.dblclick(l.selector, opts)
 }
 
@@ -151,6 +187,7 @@ func (l *Locator) SetChecked(checked bool, opts sobek.Value) error {
 
 func (l *Locator) setChecked(checked bool, opts *FrameCheckOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.setChecked(l.selector, checked, opts)
 }
 
@@ -175,6 +212,7 @@ func (l *Locator) Check(opts sobek.Value) error {
 // error, or applies slow motion.
 func (l *Locator) check(opts *FrameCheckOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.check(l.selector, opts)
 }
 
@@ -199,6 +237,7 @@ func (l *Locator) Uncheck(opts sobek.Value) error {
 // an error, or applies slow motion.
 func (l *Locator) uncheck(opts *FrameUncheckOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.uncheck(l.selector, opts)
 }
 
@@ -349,10 +388,21 @@ func (l *Locator) fill(value string, opts *FrameFillOptions) error {
 	return l.frame.fill(l.selector, value, opts)
 }
 
+// LocatorFilterOptions allows filtering a [Locator] by various criteria.
+// It's similar to [LocatorOptions] but used for filtering existing locators.
+type LocatorFilterOptions struct {
+	*LocatorOptions
+}
+
+// Filter returns a new [Locator] after applying the options to the current one.
+func (l *Locator) Filter(opts *LocatorFilterOptions) *Locator {
+	return NewLocator(l.ctx, opts.LocatorOptions, l.selector, l.frame, l.log)
+}
+
 // First will return the first child of the element matching the locator's
 // selector.
 func (l *Locator) First() *Locator {
-	return NewLocator(l.ctx, l.selector+" >> nth=0", l.frame, l.log)
+	return NewLocator(l.ctx, nil, l.selector+" >> nth=0", l.frame, l.log)
 }
 
 // Focus on the element using locator's selector with strict mode on.
@@ -402,6 +452,82 @@ func (l *Locator) getAttribute(name string, opts *FrameBaseOptions) (string, boo
 	return l.frame.getAttribute(l.selector, name, opts)
 }
 
+// GetByAltText creates and returns a new relative locator that allows locating elements by their alt text.
+func (l *Locator) GetByAltText(alt string, opts *GetByBaseOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByAltText", "fid:%s furl:%q selector:%s alt:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, alt, opts,
+	)
+
+	return l.Locator(l.frame.buildAttributeSelector("alt", alt, opts), nil)
+}
+
+// GetByLabel creates and returns a new relative locator that allows locating input elements by the text
+// of the associated `<label>` or `aria-labelledby` element, or by the `aria-label` attribute.
+func (l *Locator) GetByLabel(label string, opts *GetByBaseOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByLabel", "fid:%s furl:%q selector:%s label:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, label, opts,
+	)
+
+	return l.Locator(l.frame.buildLabelSelector(label, opts), nil)
+}
+
+// GetByPlaceholder creates and returns a new relative locator for this based on the placeholder attribute.
+func (l *Locator) GetByPlaceholder(placeholder string, opts *GetByBaseOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByPlaceholder", "fid:%s furl:%q selector:%s placeholder:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, placeholder, opts,
+	)
+
+	return l.Locator(l.frame.buildAttributeSelector("placeholder", placeholder, opts), nil)
+}
+
+// GetByRole creates and returns a new relative locator using the ARIA role and any additional options.
+func (l *Locator) GetByRole(role string, opts *GetByRoleOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByRole", "fid:%s furl:%q selector:%s role:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, role, opts,
+	)
+
+	return l.Locator(l.frame.buildRoleSelector(role, opts), nil)
+}
+
+// GetByTestID creates and returns a new relative locator based on the data-testid attribute.
+func (l *Locator) GetByTestID(testID string) *Locator {
+	l.log.Debugf(
+		"Locator:GetByTestID", "fid:%s furl:%q selector:%s testID:%q",
+		l.frame.ID(), l.frame.URL(), l.selector, testID,
+	)
+
+	return l.Locator(l.frame.buildTestIDSelector(testID), nil)
+}
+
+// GetByText creates and returns a new relative locator based on text content.
+func (l *Locator) GetByText(text string, opts *GetByBaseOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByText", "fid:%s furl:%q selector:%s text:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, text, opts,
+	)
+
+	return l.Locator(l.frame.buildTextSelector(text, opts), nil)
+}
+
+// GetByTitle creates and returns a new relative locator based on the title attribute.
+func (l *Locator) GetByTitle(title string, opts *GetByBaseOptions) *Locator {
+	l.log.Debugf(
+		"Locator:GetByTitle", "fid:%s furl:%q selector:%s title:%q opts:%+v",
+		l.frame.ID(), l.frame.URL(), l.selector, title, opts,
+	)
+
+	return l.Locator(l.frame.buildAttributeSelector("title", title, opts), nil)
+}
+
+// Locator creates and returns a new locator chained/relative to the current locator.
+func (l *Locator) Locator(selector string, opts *LocatorOptions) *Locator {
+	return NewLocator(l.ctx, opts, l.selector+" >> "+selector, l.frame, l.log)
+}
+
 // InnerHTML returns the element's inner HTML that matches
 // the locator's selector with strict mode on.
 func (l *Locator) InnerHTML(opts sobek.Value) (string, error) {
@@ -449,13 +575,13 @@ func (l *Locator) innerText(opts *FrameInnerTextOptions) (string, error) {
 // Last will return the last child of the element matching the locator's
 // selector.
 func (l *Locator) Last() *Locator {
-	return NewLocator(l.ctx, l.selector+" >> nth=-1", l.frame, l.log)
+	return NewLocator(l.ctx, nil, l.selector+" >> nth=-1", l.frame, l.log)
 }
 
 // Nth will return the nth child of the element matching the locator's
 // selector.
 func (l *Locator) Nth(nth int) *Locator {
-	return NewLocator(l.ctx, l.selector+" >> nth="+strconv.Itoa(nth), l.frame, l.log)
+	return NewLocator(l.ctx, nil, l.selector+" >> nth="+strconv.Itoa(nth), l.frame, l.log)
 }
 
 // TextContent returns the element's text content that matches
@@ -611,6 +737,7 @@ func (l *Locator) Hover(opts sobek.Value) error {
 
 func (l *Locator) hover(opts *FrameHoverOptions) error {
 	opts.Strict = true
+	opts.retry = true
 	return l.frame.hover(l.selector, opts)
 }
 
@@ -619,6 +746,7 @@ func (l *Locator) Tap(opts *FrameTapOptions) error {
 	l.log.Debugf("Locator:Tap", "fid:%s furl:%q sel:%q opts:%+v", l.frame.ID(), l.frame.URL(), l.selector, opts)
 
 	opts.Strict = true
+	opts.retry = true
 	if err := l.frame.tap(l.selector, opts); err != nil {
 		return fmt.Errorf("tapping on %q: %w", l.selector, err)
 	}
