@@ -7,32 +7,28 @@ import (
 	"testing"
 
 	"github.com/grafana/sobek"
-	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"go.k6.io/k6/internal/js/modules/k6/browser/common"
 )
 
+const (
+	frameLocatorImpl = "frameLocator"
+	locatorImpl      = "locator"
+	frameImpl        = "frame"
+	pageImpl         = "page"
+	iframeID         = "frameB"
+)
+
 func TestGetByRoleSuccess(t *testing.T) {
 	t.Parallel()
 
-	// This test all the implicit roles that are valid for the role based
-	// selector engine that is in the injectd_script.js file. Implicit roles
+	// This test all the implicit roles that are valid for the role-based
+	// selector engine that is in the injected_script.js file. Implicit roles
 	// are roles that are not explicitly defined in the HTML, but are
 	// implied by the context of the element.
 	t.Run("implicit", func(t *testing.T) {
 		t.Parallel()
 
-		tb := newTestBrowser(t, withFileServer())
-		p := tb.NewPage(nil)
-		opts := &common.FrameGotoOptions{
-			Timeout: common.DefaultTimeout,
-		}
-		_, err := p.Goto(
-			tb.staticURL("get_by_role_implicit.html"),
-			opts,
-		)
-		require.NoError(t, err)
-
 		tests := []struct {
 			name         string
 			role         string
@@ -441,38 +437,46 @@ func TestGetByRoleSuccess(t *testing.T) {
 			t.Run(tt.name, func(t *testing.T) {
 				t.Parallel()
 
-				l := p.GetByRole(tt.role, tt.opts)
-				c, err := l.Count()
-				assert.NoError(t, err)
-				assert.Equal(t, tt.expected, c)
+				tb := newTestBrowser(t, withFileServer())
+				staticURL := tb.staticURL("get_by_role_implicit.html")
+				tb.withIFrameURL(staticURL, iframeID)
+				p := tb.NewPage(nil)
 
-				if tt.expectedText != "" {
-					text, _, err := l.TextContent(sobek.Undefined())
-					assert.NoError(t, err)
-					assert.Equal(t, tt.expectedText, text)
+				getByRoleImplementations := getByImplementationsOf[interface {
+					GetByRole(role string, opts *common.GetByRoleOptions) *common.Locator
+				}](p)
+
+				for implName, impl := range getByRoleImplementations {
+					t.Run(implName, func(t *testing.T) { //nolint:paralleltest
+						if implName == frameLocatorImpl {
+							tb.GotoPage(p, tb.url("/iframe"))
+						} else {
+							tb.GotoPage(p, staticURL)
+						}
+
+						l := impl.GetByRole(tt.role, tt.opts)
+						c, err := l.Count()
+						require.NoError(t, err)
+						require.Equal(t, tt.expected, c)
+
+						if tt.expectedText != "" {
+							text, _, err := l.TextContent(sobek.Undefined())
+							require.NoError(t, err)
+							require.Equal(t, tt.expectedText, text)
+						}
+					})
 				}
 			})
 		}
 	})
 
-	// This test all the explicit roles that are valid for the role based
-	// selector engine that is in the injectd_script.js file. Explicit roles
+	// This test all the explicit roles that are valid for the role-based
+	// selector engine that is in the injected_script.js file. Explicit roles
 	// are roles that are explicitly defined in the HTML using the correct
 	// role attribute.
 	t.Run("explicit", func(t *testing.T) {
 		t.Parallel()
 
-		tb := newTestBrowser(t, withFileServer())
-		p := tb.NewPage(nil)
-		opts := &common.FrameGotoOptions{
-			Timeout: common.DefaultTimeout,
-		}
-		_, err := p.Goto(
-			tb.staticURL("get_by_role_explicit.html"),
-			opts,
-		)
-		require.NoError(t, err)
-
 		tests := []struct {
 			role         string
 			expected     int
@@ -497,8 +501,6 @@ func TestGetByRoleSuccess(t *testing.T) {
 			{role: "deletion", expected: 1, expectedText: "Deletion"},
 			{role: "dialog", expected: 1, expectedText: "Dialog"},
 			{role: "directory", expected: 1, expectedText: "Directory"},
-			// The original document plus the one within the html <section>
-			{role: "document", expected: 2, expectedText: ""},
 			{role: "emphasis", expected: 1, expectedText: "Emphasis"},
 			{role: "feed", expected: 1, expectedText: "Feed"},
 			{role: "figure", expected: 1, expectedText: "Figure"},
@@ -567,37 +569,82 @@ func TestGetByRoleSuccess(t *testing.T) {
 			t.Run(tt.role, func(t *testing.T) {
 				t.Parallel()
 
-				l := p.GetByRole(tt.role, nil)
-				c, err := l.Count()
-				assert.NoError(t, err)
-				assert.Equal(t, tt.expected, c)
+				tb := newTestBrowser(t, withFileServer())
+				staticURL := tb.staticURL("get_by_role_explicit.html")
+				tb.withIFrameURL(staticURL, iframeID)
+				p := tb.NewPage(nil)
 
-				if tt.expectedText != "" {
-					text, err := l.InnerText(sobek.Undefined())
-					assert.NoError(t, err)
-					assert.Equal(t, tt.expectedText, text)
+				getByRoleImplementations := getByImplementationsOf[interface {
+					GetByRole(role string, opts *common.GetByRoleOptions) *common.Locator
+				}](p)
+
+				for implName, impl := range getByRoleImplementations {
+					t.Run(implName, func(t *testing.T) { //nolint:paralleltest
+						if implName == frameLocatorImpl {
+							tb.GotoPage(p, tb.url("/iframe"))
+						} else {
+							tb.GotoPage(p, staticURL)
+						}
+
+						l := impl.GetByRole(tt.role, nil)
+						c, err := l.Count()
+						require.NoError(t, err)
+						require.Equal(t, tt.expected, c)
+
+						if tt.expectedText != "" {
+							text, err := l.InnerText(sobek.Undefined())
+							require.NoError(t, err)
+							require.Equal(t, tt.expectedText, text)
+						}
+					})
 				}
 			})
 		}
+
+		// We test the 'document' role independently, because the expectations
+		// for each getByRole implementation (page, frame, locator) are different:
+		t.Run("document", func(t *testing.T) {
+			t.Parallel()
+
+			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_role_explicit.html")
+			tb.withIFrameURL(staticURL, iframeID)
+			p := tb.NewPage(nil)
+
+			getByRoleImplementations := getByImplementationsOf[interface {
+				GetByRole(role string, opts *common.GetByRoleOptions) *common.Locator
+			}](p)
+
+			expectedByImplementation := map[string]int{
+				pageImpl:         2,
+				frameImpl:        2,
+				locatorImpl:      1,
+				frameLocatorImpl: 2,
+			}
+
+			for implName, impl := range getByRoleImplementations {
+				t.Run(implName, func(t *testing.T) { //nolint:paralleltest
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByRole("document", nil)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, expectedByImplementation[implName], c)
+				})
+			}
+		})
 	})
 
-	// This tests all the options, and different attributes (such as explicit
+	// This tests all the options and different attributes (such as explicit
 	// aria attributes vs the text value of an element) that can be used in
 	// the DOM with the same role.
 	t.Run("edge_cases", func(t *testing.T) {
 		t.Parallel()
 
-		tb := newTestBrowser(t, withFileServer())
-		p := tb.NewPage(nil)
-		opts := &common.FrameGotoOptions{
-			Timeout: common.DefaultTimeout,
-		}
-		_, err := p.Goto(
-			tb.staticURL("get_by_role_edge_cases.html"),
-			opts,
-		)
-		require.NoError(t, err)
-
 		tests := []struct {
 			name         string
 			role         string
@@ -724,15 +771,34 @@ func TestGetByRoleSuccess(t *testing.T) {
 			t.Run(tt.name, func(t *testing.T) {
 				t.Parallel()
 
-				l := p.GetByRole(tt.role, tt.opts)
-				c, err := l.Count()
-				assert.NoError(t, err)
-				assert.Equal(t, tt.expected, c)
+				tb := newTestBrowser(t, withFileServer())
+				staticURL := tb.staticURL("get_by_role_edge_cases.html")
+				tb.withIFrameURL(staticURL, iframeID)
+				p := tb.NewPage(nil)
 
-				if tt.expectedText != "" {
-					text, err := l.InnerText(sobek.Undefined())
-					assert.NoError(t, err)
-					assert.Equal(t, tt.expectedText, text)
+				getByRoleImplementations := getByImplementationsOf[interface {
+					GetByRole(role string, opts *common.GetByRoleOptions) *common.Locator
+				}](p)
+
+				for implName, impl := range getByRoleImplementations {
+					t.Run(implName, func(t *testing.T) { //nolint:paralleltest
+						if implName == frameLocatorImpl {
+							tb.GotoPage(p, tb.url("/iframe"))
+						} else {
+							tb.GotoPage(p, staticURL)
+						}
+
+						l := impl.GetByRole(tt.role, tt.opts)
+						c, err := l.Count()
+						require.NoError(t, err)
+						require.Equal(t, tt.expected, c)
+
+						if tt.expectedText != "" {
+							text, err := l.InnerText(sobek.Undefined())
+							require.NoError(t, err)
+							require.Equal(t, tt.expectedText, text)
+						}
+					})
 				}
 			})
 		}
@@ -766,19 +832,27 @@ func TestGetByRoleFailure(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_role.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_role.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByRole(tt.role, tt.opts)
-			_, err = l.Count()
-			assert.ErrorContains(t, err, tt.expectedError)
+			getByRoleImplementations := getByImplementationsOf[interface {
+				GetByRole(role string, opts *common.GetByRoleOptions) *common.Locator
+			}](p)
+
+			for implName, impl := range getByRoleImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByRole(tt.role, tt.opts)
+					_, err := l.Count()
+					require.ErrorContains(t, err, tt.expectedError)
+				})
+			}
 		})
 	}
 }
@@ -835,20 +909,28 @@ func TestGetByAltTextSuccess(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_alt_text.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_alt_text.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByAltText(tt.alt, tt.opts)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByAltTextImplementations := getByImplementationsOf[interface {
+				GetByAltText(alt string, opts *common.GetByBaseOptions) *common.Locator
+			}](p)
+
+			for implName, impl := range getByAltTextImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByAltText(tt.alt, tt.opts)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+				})
+			}
 		})
 	}
 }
@@ -867,14 +949,14 @@ func TestGetByLabelSuccess(t *testing.T) {
 			"missing_label",
 			"",
 			nil,
-			3,
+			4,
 		},
 		{
 			// matches on all the elements with a label.
 			"empty_string",
 			`""`,
 			nil,
-			3,
+			4,
 		},
 		{
 			"aria_label",
@@ -900,26 +982,40 @@ func TestGetByLabelSuccess(t *testing.T) {
 			nil,
 			1,
 		},
+		{
+			"aria_labelledby",
+			`"I agree to the Terms and Conditions."`,
+			nil,
+			1,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_label.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_label.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByLabel(tt.label, tt.opts)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByLabelImplementations := getByImplementationsOf[interface {
+				GetByLabel(label string, opts *common.GetByBaseOptions) *common.Locator
+			}](p)
+
+			for implName, impl := range getByLabelImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByLabel(tt.label, tt.opts)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+				})
+			}
 		})
 	}
 }
@@ -994,20 +1090,28 @@ func TestGetByPlaceholderSuccess(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_placeholder.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_placeholder.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByPlaceholder(tt.placeholder, tt.opts)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByPlaceholderImplementations := getByImplementationsOf[interface {
+				GetByPlaceholder(placeholder string, opts *common.GetByBaseOptions) *common.Locator
+			}](p)
+
+			for implName, impl := range getByPlaceholderImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByPlaceholder(tt.placeholder, tt.opts)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+				})
+			}
 		})
 	}
 }
@@ -1082,20 +1186,28 @@ func TestGetByTitleSuccess(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_title.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_title.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByTitle(tt.title, tt.opts)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByTitleImplementations := getByImplementationsOf[interface {
+				GetByTitle(title string, opts *common.GetByBaseOptions) *common.Locator
+			}](p)
+
+			for implName, impl := range getByTitleImplementations {
+				if implName == frameLocatorImpl {
+					tb.GotoPage(p, tb.url("/iframe"))
+				} else {
+					tb.GotoPage(p, staticURL)
+				}
+
+				t.Run(implName, func(t *testing.T) {
+					l := impl.GetByTitle(tt.title, tt.opts)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+				})
+			}
 		})
 	}
 }
@@ -1163,25 +1275,33 @@ func TestGetByTestIDSuccess(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_testid.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_testid.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByTestID(tt.testID)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByTestIDImplementations := getByImplementationsOf[interface {
+				GetByTestID(testID string) *common.Locator
+			}](p)
 
-			if tt.expected > 0 && tt.expectedText != "" {
-				text, err := l.InnerText(sobek.Undefined())
-				assert.NoError(t, err)
-				assert.Equal(t, tt.expectedText, text)
+			for implName, impl := range getByTestIDImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByTestID(tt.testID)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+
+					if tt.expected > 0 && tt.expectedText != "" {
+						text, err := l.InnerText(sobek.Undefined())
+						require.NoError(t, err)
+						require.Equal(t, tt.expectedText, text)
+					}
+				})
 			}
 		})
 	}
@@ -1273,25 +1393,33 @@ func TestGetByTextSuccess(t *testing.T) {
 			t.Parallel()
 
 			tb := newTestBrowser(t, withFileServer())
+			staticURL := tb.staticURL("get_by_text.html")
+			tb.withIFrameURL(staticURL, iframeID)
 			p := tb.NewPage(nil)
-			opts := &common.FrameGotoOptions{
-				Timeout: common.DefaultTimeout,
-			}
-			_, err := p.Goto(
-				tb.staticURL("get_by_text.html"),
-				opts,
-			)
-			require.NoError(t, err)
 
-			l := p.GetByText(tt.text, tt.opts)
-			c, err := l.Count()
-			assert.NoError(t, err)
-			assert.Equal(t, tt.expected, c)
+			getByTextImplementations := getByImplementationsOf[interface {
+				GetByText(text string, opts *common.GetByBaseOptions) *common.Locator
+			}](p)
 
-			if tt.expected > 0 && tt.expectedText != "" {
-				text, err := l.InnerText(sobek.Undefined())
-				assert.NoError(t, err)
-				assert.Equal(t, tt.expectedText, text)
+			for implName, impl := range getByTextImplementations {
+				t.Run(implName, func(t *testing.T) {
+					if implName == frameLocatorImpl {
+						tb.GotoPage(p, tb.url("/iframe"))
+					} else {
+						tb.GotoPage(p, staticURL)
+					}
+
+					l := impl.GetByText(tt.text, tt.opts)
+					c, err := l.Count()
+					require.NoError(t, err)
+					require.Equal(t, tt.expected, c)
+
+					if tt.expected > 0 && tt.expectedText != "" {
+						text, err := l.InnerText(sobek.Undefined())
+						require.NoError(t, err)
+						require.Equal(t, tt.expectedText, text)
+					}
+				})
 			}
 		})
 	}
@@ -1308,41 +1436,55 @@ func TestGetByNullHandling(t *testing.T) {
 	tb.vu.SetVar(t, "page", &sobek.Object{})
 	_, err := tb.vu.RunAsync(t, `
 		page = await browser.newPage();
-	`)
+		frame = page.mainFrame();
+		locator = page.locator(':root');
+		frameLocator = page.locator('#%s').contentFrame();
+	`, iframeID)
 	require.NoError(t, err)
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByRole().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'role'")
+	for _, getByImpl := range []string{pageImpl, frameImpl, locatorImpl, frameLocatorImpl} {
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByRole().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'role'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByAltText().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'altText'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByAltText().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'altText'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByLabel().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'label'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByLabel().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'label'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByPlaceholder().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'placeholder'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByPlaceholder().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'placeholder'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByTitle().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'title'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByTitle().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'title'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByTestId().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'testId'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByTestId().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'testId'")
 
-	_, err = tb.vu.RunAsync(t, `
-		await page.getByText().click();
-	`)
-	require.ErrorContains(t, err, "missing required argument 'text'")
+		_, err = tb.vu.RunAsync(t, `
+		await %s.getByText().click();
+	`, getByImpl)
+		require.ErrorContains(t, err, "missing required argument 'text'")
+	}
+}
+
+func getByImplementationsOf[T any](p *common.Page) map[string]T {
+	return map[string]T{
+		pageImpl:         any(p).(T),
+		frameImpl:        any(p.MainFrame()).(T),
+		locatorImpl:      any(p.Locator(":root", nil)).(T),
+		frameLocatorImpl: any(p.Locator("#"+iframeID, nil).ContentFrame()).(T),
+	}
 }
