@@ -138,7 +138,7 @@ func TestLifecycleWaitForNavigation(t *testing.T) {
 					Timeout:   1000 * time.Millisecond,
 					WaitUntil: common.LifecycleEventNetworkIdle,
 				}
-				_, err = p.WaitForNavigation(opts)
+				_, err = p.WaitForNavigation(opts, nil)
 
 				return err
 			},
@@ -147,7 +147,6 @@ func TestLifecycleWaitForNavigation(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
@@ -183,7 +182,7 @@ func TestLifecycleWaitForNavigation(t *testing.T) {
 						Timeout:   30000 * time.Millisecond,
 						WaitUntil: tt.waitUntil,
 					}
-					_, err := p.WaitForNavigation(opts)
+					_, err := p.WaitForNavigation(opts, nil)
 					return err
 				}
 				click := func() error {
@@ -296,7 +295,6 @@ func TestLifecycleWaitForLoadState(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
@@ -391,7 +389,6 @@ func TestLifecycleReload(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
@@ -500,7 +497,6 @@ func TestLifecycleGotoWithSubFrame(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
@@ -578,7 +574,6 @@ func TestLifecycleGoto(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			t.Parallel()
 
