@@ -9,6 +9,7 @@ import (
 	"io"
 	"net"
 	"net/http"
+	"net/http/httptest"
 	"runtime"
 	"slices"
 	"strconv"
@@ -530,17 +531,17 @@ func TestPageInputValue(t *testing.T) {
 	inputValue, err := p.InputValue("input", common.NewFrameInputValueOptions(p.MainFrame().Timeout()))
 	require.NoError(t, err)
 	got, want := inputValue, "hello1"
-	assert.Equal(t, got, want)
+	assert.Equal(t, want, got)
 
 	inputValue, err = p.InputValue("select", common.NewFrameInputValueOptions(p.MainFrame().Timeout()))
 	require.NoError(t, err)
 	got, want = inputValue, "hello2"
-	assert.Equal(t, got, want)
+	assert.Equal(t, want, got)
 
 	inputValue, err = p.InputValue("textarea", common.NewFrameInputValueOptions(p.MainFrame().Timeout()))
 	require.NoError(t, err)
 	got, want = inputValue, "hello3"
-	assert.Equal(t, got, want)
+	assert.Equal(t, want, got)
 }
 
 // test for: https://go.k6.io/k6/js/modules/k6/browser/issues/132
@@ -818,7 +819,7 @@ func TestPageWaitForFunction(t *testing.T) {
 		tb.logCache.contains("ok: null")
 
 		p := tb.vu.RunPromise(t, `return await page.evaluate(() => window._arg);`)
-		require.Equal(t, p.State(), sobek.PromiseStateFulfilled)
+		require.Equal(t, sobek.PromiseStateFulfilled, p.State())
 		assert.Equal(t, "raf_arg", p.Result().String())
 	})
 
@@ -842,7 +843,7 @@ func TestPageWaitForFunction(t *testing.T) {
 		tb.logCache.contains("ok: null")
 
 		p := tb.vu.RunPromise(t, `return await page.evaluate(() => window._args);`)
-		require.Equal(t, p.State(), sobek.PromiseStateFulfilled)
+		require.Equal(t, sobek.PromiseStateFulfilled, p.State())
 		var gotArgs []int
 		_ = tb.vu.Runtime().ExportTo(p.Result(), &gotArgs)
 		assert.Equal(t, args, gotArgs)
@@ -1409,7 +1410,7 @@ func TestPageWaitForSelector(t *testing.T) {
 			url:  "wait_for.html",
 			// set a timeout smaller than the time
 			// it takes the element to show up
-			customTimeout: time.Millisecond,
+			customTimeout: time.Nanosecond,
 			selector:      "#my-div",
 			errAssert: func(t *testing.T, e error) {
 				t.Helper()
@@ -2800,200 +2801,638 @@ func TestWaitForNavigationWithURL(t *testing.T) {
 	tb.vu.ActivateVU()
 	tb.vu.StartIteration(t)
 
-	got := tb.vu.RunPromise(t, `
-		const page = await browser.newPage();
-		const testURL = '%s';
-
-		try {
-			await page.goto(testURL);
-
-			// Test exact URL match
-			await Promise.all([
-				page.waitForNavigation({ url: '%s' }),
-				page.locator('#page1').click()
-			]);
-			let currentURL = page.url();
-			if (!currentURL.endsWith('page1.html')) {
-				throw new Error('Expected to navigate to page1.html but got ' + currentURL);
-			}
-
-			await page.goto(testURL);
-
-			// Test regex pattern - matches any page with .html extension
-			await Promise.all([
-				page.waitForNavigation({ url: /.*\.html$/ }),
-				page.locator('#page2').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('.html')) {
-				throw new Error('Expected URL to end with .html but got ' + currentURL);
-			}
-
-			await page.goto(testURL);
-
-			// Test timeout when URL doesn't match
-			let timedOut = false;
-			try {
-				await Promise.all([
-					page.waitForNavigation({ url: /.*nonexistent.html$/, timeout: 500 }),
-					page.locator('#page1').click()  // This goes to page1.html, not nonexistent.html
-				]);
-			} catch (error) {
-				if (error.toString().includes('waiting for navigation')) {
-					timedOut = true;
-				} else {
-					throw error;
-				}
-			}
-			if (!timedOut) {
-				throw new Error('Expected timeout error when URL does not match');
-			}
-
-			await page.goto(testURL);
-
-			// Test empty pattern (matches any navigation)
-			await Promise.all([
-				page.waitForNavigation({ url: '' }),
-				page.locator('#page2').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('page2.html')) {
-				throw new Error('Expected empty pattern to match any navigation but got ' + currentURL);
-			}
-		} finally {
-			// Must call close() which will clean up the taskqueue.
-			await page.close();
-		}
-	`,
-		tb.staticURL("waitfornavigation_test.html"),
-		tb.staticURL("page1.html"),
-	)
-	assert.Equal(t, sobek.Undefined(), got.Result())
-}
-
-func TestWaitForNavigationWithURL_RegexFailure(t *testing.T) {
-	t.Parallel()
-
-	tb := newTestBrowser(t, withFileServer())
-	tb.vu.ActivateVU()
-	tb.vu.StartIteration(t)
-
+	// Setup
+	tb.vu.SetVar(t, "page", &sobek.Object{})
+	tb.vu.SetVar(t, "testURL", tb.staticURL("waitfornavigation_test.html"))
+	tb.vu.SetVar(t, "page1URL", tb.staticURL("page1.html"))
 	_, err := tb.vu.RunAsync(t, `
-		const page = await browser.newPage();
-		await page.goto('%s');
+			page = await browser.newPage();
+		`)
+	require.NoError(t, err)
+
+	// Test exact URL match
+	got := tb.vu.RunPromise(t, `
+		await page.goto(testURL);
+
+		await Promise.all([
+			page.waitForNavigation({ url: page1URL }),
+			page.locator('#page1').click()
+		]);
+		return page.url();
+	`,
+	)
+	assert.Equal(t, tb.staticURL("page1.html"), got.Result().String())
+
+	// Test regex pattern - matches any page with .html extension
+	got = tb.vu.RunPromise(t, `
+		await page.goto(testURL);
+
+		await Promise.all([
+			page.waitForNavigation({ url: /.*2\.html$/ }),
+			page.locator('#page2').click()
+		]);
+		return page.url();
+	`,
+	)
+	assert.Equal(t, tb.staticURL("page2.html"), got.Result().String())
+
+	// Test timeout when URL doesn't match
+	_, err = tb.vu.RunAsync(t, `
+		await page.goto(testURL);
+
+		await Promise.all([
+			page.waitForNavigation({ url: /.*nonexistent.html$/, timeout: 500 }),
+			page.locator('#page1').click()  // This goes to page1.html, not nonexistent.html
+		]);
+	`,
+	)
+	assert.ErrorContains(t, err, "timed out after 500ms")
+
+	// Test empty pattern (matches any navigation)
+	got = tb.vu.RunPromise(t, `
+		await page.goto(testURL);
+
+		await Promise.all([
+			page.waitForNavigation({ url: '' }),
+			page.locator('#page2').click()
+		]);
+		return page.url();
+	`,
+	)
+	assert.Equal(t, tb.staticURL("page2.html"), got.Result().String())
+
+	// Test regex pattern with invalid regex
+	_, err = tb.vu.RunAsync(t, `
+		await page.goto(testURL);
 
 		await Promise.all([
 			page.waitForNavigation({ url: /^.*/my_messages.*$/ }),
 			page.locator('#page2').click()
 		]);
 	`,
-		tb.staticURL("waitfornavigation_test.html"),
 	)
 	assert.ErrorContains(t, err, "Unexpected token *")
 }
 
-func TestWaitForURL(t *testing.T) {
+func TestPageWaitForURLSuccess(t *testing.T) {
+	t.Parallel()
+
 	if runtime.GOOS == "windows" {
 		t.Skip("Skipped due to https://github.com/grafana/k6/issues/4937")
 	}
 
+	tests := []struct {
+		name     string
+		code     string
+		expected []string
+	}{
+		{
+			name:     "when_already_at_matching_url",
+			code:     `await page.waitForURL(/.*waitfornavigation_test\.html$/);`,
+			expected: []string{"waitfornavigation_test.html"},
+		},
+		{
+			name: "exact_url_match",
+			code: `
+				await Promise.all([
+					page.waitForURL(page1URL),
+					page.locator('#page1').click()
+				]);
+			`,
+			expected: []string{"page1.html"},
+		},
+		{
+			name: "regex_pattern_match",
+			code: `
+				await Promise.all([
+					page.waitForURL(/.*2\.html$/),
+					page.locator('#page2').click()
+				]);
+			`,
+			expected: []string{"page2.html"},
+		},
+		{
+			name: "empty_pattern_match",
+			code: `
+				await Promise.all([
+					page.waitForURL(''),
+					page.locator('#page2').click()
+				]);
+			`,
+			expected: []string{"page2.html", "waitfornavigation_test.html"},
+		},
+		{
+			name: "waitUntil_domcontentloaded",
+			code: `
+				await Promise.all([
+					page.waitForURL(/.*page1\.html$/, { waitUntil: 'domcontentloaded' }),
+					page.locator('#page1').click()
+				]);
+			`,
+			expected: []string{"page1.html"},
+		},
+		{
+			name: "already_at_url_with_regex_pattern",
+			code: `
+				await page.waitForURL(/.*\/waitfornavigation_test\.html$/);
+			`,
+			expected: []string{"waitfornavigation_test.html"},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			t.Parallel()
+
+			// Setup
+			tb := newTestBrowser(t, withFileServer())
+			tb.vu.ActivateVU()
+			tb.vu.StartIteration(t)
+
+			tb.vu.SetVar(t, "page", &sobek.Object{})
+			tb.vu.SetVar(t, "testURL", tb.staticURL("waitfornavigation_test.html"))
+			tb.vu.SetVar(t, "page1URL", tb.staticURL("page1.html"))
+			_, err := tb.vu.RunAsync(t, `
+					page = await browser.newPage();
+				`)
+			require.NoError(t, err)
+
+			// test logic
+			code := fmt.Sprintf(`
+			await page.goto(testURL);
+
+			%s
+			
+			return page.url();`, tt.code)
+
+			result := tb.vu.RunPromise(t, code)
+			got := strings.ReplaceAll(result.Result().String(), tb.staticURL(""), "")
+			assert.Contains(t, tt.expected, got)
+		})
+	}
+}
+
+func TestPageWaitForURLFailure(t *testing.T) {
 	t.Parallel()
+	if runtime.GOOS == "windows" {
+		t.Skip("Skipped due to https://github.com/grafana/k6/issues/4937")
+	}
 
-	tb := newTestBrowser(t, withFileServer())
-	tb.vu.ActivateVU()
-	tb.vu.StartIteration(t)
-
-	got := tb.vu.RunPromise(t, `
-		const page = await browser.newPage();
-		const testURL = '%s';
-
-		try {
-			// Test when already at matching URL (should just wait for load state)
-			await page.goto('%s');
-			await page.waitForURL(/.*page1\.html$/);
-			let currentURL = page.url();
-			if (!currentURL.endsWith('page1.html')) {
-				throw new Error('Expected to stay at page1.html but got ' + currentURL);
-			}
-
-			// Test exact URL match with navigation
-			await page.goto(testURL);
-			await Promise.all([
-				page.waitForURL('%s'),
-				page.locator('#page1').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('page1.html')) {
-				throw new Error('Expected to navigate to page1.html but got ' + currentURL);
-			}
-
-			// Test regex pattern - matches any page with .html extension
-			await page.goto(testURL);
-			await Promise.all([
-				page.waitForURL(/.*\.html$/),
-				page.locator('#page2').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('.html')) {
-				throw new Error('Expected URL to end with .html but got ' + currentURL);
-			}
-
-			// Test timeout when URL doesn't match
-			await page.goto(testURL);
-			let timedOut = false;
-			try {
+	tests := []struct {
+		name     string
+		code     string
+		expected string
+	}{
+		{
+			name: "timeout_on_mismatched_url",
+			code: `
 				await Promise.all([
 					page.waitForURL(/.*nonexistent\.html$/, { timeout: 500 }),
 					page.locator('#page1').click()  // This goes to page1.html, not nonexistent.html
 				]);
-			} catch (error) {
-				if (error.toString().includes('waiting for navigation')) {
-					timedOut = true;
-				} else {
-					throw error;
-				}
-			}
-			if (!timedOut) {
-				throw new Error('Expected timeout error when URL does not match');
-			}
+			`,
+			expected: "timed out after 500ms",
+		},
+		{
+			name: "missing_required_argument",
+			code: `
+				await page.waitForURL();
+			`,
+			expected: "missing required argument 'url'",
+		},
+	}
 
-			// Test empty pattern (matches any navigation)
-			await page.goto(testURL);
-			await Promise.all([
-				page.waitForURL(''),
-				page.locator('#page2').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('page2.html') && !currentURL.endsWith('waitfornavigation_test.html')) {
-				throw new Error('Expected empty pattern to match any navigation but got ' + currentURL);
-			}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			t.Parallel()
 
-			// Test waitUntil option
-			await page.goto(testURL);
-			await Promise.all([
-				page.waitForURL(/.*page1\.html$/, { waitUntil: 'domcontentloaded' }),
-				page.locator('#page1').click()
-			]);
-			currentURL = page.url();
-			if (!currentURL.endsWith('page1.html')) {
-				throw new Error('Expected to navigate to page1.html with domcontentloaded but got ' + currentURL);
-			}
+			tb := newTestBrowser(t, withFileServer())
+			tb.vu.ActivateVU()
+			tb.vu.StartIteration(t)
 
-			// Test when already at URL with regex pattern
+			// Setup
+			tb.vu.SetVar(t, "page", &sobek.Object{})
+			tb.vu.SetVar(t, "testURL", tb.staticURL("waitfornavigation_test.html"))
+			_, err := tb.vu.RunAsync(t, `
+					page = await browser.newPage();
+				`)
+			require.NoError(t, err)
+
+			code := fmt.Sprintf(`
 			await page.goto(testURL);
-			await page.waitForURL(/.*\/waitfornavigation_test\.html$/);
-			currentURL = page.url();
-			if (!currentURL.endsWith('waitfornavigation_test.html')) {
-				throw new Error('Expected to stay at waitfornavigation_test.html but got ' + currentURL);
-			}
-		} finally {
-			// Must call close() which will clean up the taskqueue.
-			await page.close();
-		}
-	`,
-		tb.staticURL("waitfornavigation_test.html"),
-		tb.staticURL("page1.html"),
-		tb.staticURL("page1.html"),
-	)
-	assert.Equal(t, sobek.Undefined(), got.Result())
+
+			%s`, tt.code)
+
+			_, err = tb.vu.RunAsync(t, code)
+			assert.ErrorContains(t, err, tt.expected)
+		})
+	}
+}
+
+func TestPageWaitForResponse(t *testing.T) {
+	t.Parallel()
+
+	t.Run("ok/correct_response_matches", func(t *testing.T) {
+		t.Parallel()
+
+		tb := newTestBrowser(t, withHTTPServer())
+		tb.withHandler("/api/users", func(w http.ResponseWriter, r *http.Request) {
+			_, err := fmt.Fprintf(w, `{"users": []}`)
+			require.NoError(t, err)
+		})
+		tb.withHandler("/api/posts", func(w http.ResponseWriter, r *http.Request) {
+			_, err := fmt.Fprintf(w, `{"posts": []}`)
+			require.NoError(t, err)
+		})
+		tb.withHandler("/page", func(w http.ResponseWriter, _ *http.Request) {
+			_, err := fmt.Fprintf(w, `
+				<!doctype html>
+				<html><body><script>
+					setTimeout(() => fetch('/api/posts'), 50);
+					setTimeout(() => fetch('/api/users'), 100);
+				</script></body></html>
+			`)
+			require.NoError(t, err)
+		})
+
+		p := tb.NewPage(nil)
+
+		gotoPage := func() error {
+			_, err := p.Goto(tb.url("/page"), &common.FrameGotoOptions{
+				WaitUntil: common.LifecycleEventDOMContentLoad,
+				Timeout:   common.DefaultTimeout,
+			})
+			return err
+		}
+
+		waitForUsers := func() error {
+			opts := common.NewPageWaitForResponseOptions(p.Timeout())
+			mockRegexChecker := func(pattern, url string) (bool, error) {
+				return strings.Contains(url, "/users"), nil
+			}
+			resp, err := p.WaitForResponse(".*users.*", opts, mockRegexChecker)
+			if err != nil {
+				return err
+			}
+			require.Contains(t, resp.URL(), "/users")
+			return nil
+		}
+
+		err := tb.run(tb.context(), gotoPage, waitForUsers)
+		require.NoError(t, err)
+	})
+
+	t.Run("err/canceled", func(t *testing.T) {
+		t.Parallel()
+		tb := newTestBrowser(t)
+		p := tb.NewPage(nil)
+		opts := common.NewPageWaitForResponseOptions(p.Timeout())
+		go tb.cancelContext()
+		<-tb.context().Done()
+
+		mockRegexChecker := func(pattern, url string) (bool, error) {
+			return strings.Contains(url, "page"), nil
+		}
+
+		_, err := p.WaitForResponse("/page", opts, mockRegexChecker)
+		require.ErrorContains(t, err, "canceled")
+	})
+
+	t.Run("err/timeout", func(t *testing.T) {
+		t.Parallel()
+
+		tb := newTestBrowser(t, withHTTPServer())
+
+		p := tb.NewPage(nil)
+
+		waitForApi := func() error {
+			opts := common.NewPageWaitForResponseOptions(500 * time.Millisecond)
+			mockRegexChecker := func(pattern, url string) (bool, error) {
+				return strings.Contains(url, "/api"), nil
+			}
+
+			resp, err := p.WaitForResponse(".*api.*", opts, mockRegexChecker)
+			if err != nil {
+				return err
+			}
+			require.NotNil(t, resp)
+			require.Contains(t, resp.URL(), "/api")
+			return nil
+		}
+
+		err := tb.run(tb.context(), waitForApi)
+		require.ErrorContains(t, err, "waiting for response")
+	})
+}
+
+// TestClickInNestedFramesCORS tests clicking on buttons within nested frames
+// which are from different origins. At the end of the test the counter in
+// each frame should be "1".
+func TestClickInNestedFramesCORS(t *testing.T) {
+	t.Parallel()
+
+	// Origin C: innermost frame with counter button and nested same-origin iframe
+	originCHTML := `<!DOCTYPE html>
+	<html>
+	<head></head>
+	<body>
+	  <p>Counter: <span id="count">0</span></p>
+	  <button id="increment">Increment Counter</button>
+	  <iframe id="frameD" src="/innerC" width="300" height="100"></iframe>
+	  <script>
+		let count = 0;
+		document.getElementById('increment').addEventListener('click', () => {
+		  count++;
+		  document.getElementById('count').textContent = count;
+		});
+	  </script>
+	</body>
+	</html>`
+
+	// Nested same-origin frame content served at /innerC on origin C
+	innerCHTML := `<!DOCTYPE html>
+	<html>
+	<head></head>
+	<body>
+	  <p>Counter D: <span id="countD">0</span></p>
+	  <button id="incrementD">Increment Counter D</button>
+	  <script>
+		let countD = 0;
+		document.getElementById('incrementD').addEventListener('click', () => {
+		  countD++;
+		  document.getElementById('countD').textContent = countD;
+		});
+	  </script>
+	</body>
+	</html>`
+
+	// Nested same-origin frame content served at /innerA on origin A
+	innerAHTML := `<!DOCTYPE html>
+	<html>
+	<head></head>
+	<body>
+	  <p>Counter A2: <span id="countA2">0</span></p>
+	  <button id="incrementA2">Increment Counter A2</button>
+	  <script>
+		let countA2 = 0;
+		document.getElementById('incrementA2').addEventListener('click', () => {
+		  countA2++;
+		  document.getElementById('countA2').textContent = countA2;
+		});
+	  </script>
+	</body>
+	</html>`
+
+	// Server for origin C
+	muxC := http.NewServeMux()
+	muxC.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Set("Content-Type", "text/html; charset=utf-8")
+		_, err := w.Write([]byte(originCHTML))
+		require.NoError(t, err)
+	})
+	muxC.HandleFunc("/innerC", func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Set("Content-Type", "text/html; charset=utf-8")
+		_, err := w.Write([]byte(innerCHTML))
+		require.NoError(t, err)
+	})
+	srvC := httptest.NewServer(muxC)
+	t.Cleanup(func() {
+		srvC.Close()
+	})
+
+	// Origin B: intermediate frame embedding origin C + own counter (with dynamic C URL)
+	originBHTML := fmt.Sprintf(`<!DOCTYPE html>
+	<html>
+	<head></head>
+	<body>
+	  <p>Counter B: <span id="countB">0</span></p>
+	  <button id="incrementB">Increment Counter B</button>
+	  <iframe id="frameC" src="%s" width="400" height="200"></iframe>
+	  <script>
+		let countB = 0;
+		document.getElementById('incrementB').addEventListener('click', () => {
+		  countB++;
+		  document.getElementById('countB').textContent = countB;
+		});
+	  </script>
+	</body>
+	</html>`, srvC.URL)
+
+	// Server for origin B
+	muxB := http.NewServeMux()
+	muxB.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Set("Content-Type", "text/html; charset=utf-8")
+		_, err := w.Write([]byte(originBHTML))
+		require.NoError(t, err)
+	})
+	srvB := httptest.NewServer(muxB)
+	t.Cleanup(func() {
+		srvB.Close()
+	})
+
+	// Origin A: main page embedding origin B and same-origin frame A (with dynamic B URL)
+	originAHTML := fmt.Sprintf(`<!DOCTYPE html>
+	<html>
+	<head></head>
+	<body>
+	  <p>Counter A: <span id="countA">0</span></p>
+	  <button id="incrementA">Increment Counter A</button>
+	  <iframe id="frameA" src="/innerA" width="300" height="150" style="display: block; margin: 10px auto;"></iframe>
+	  <iframe id="frameB" src="%s" width="450" height="300" style="display: block; margin: 10px auto;"></iframe>
+	  <script>
+		let countA = 0;
+		document.getElementById('incrementA').addEventListener('click', () => {
+		  countA++;
+		  document.getElementById('countA').textContent = countA;
+		});
+	  </script>
+	</body>
+	</html>`, srvB.URL)
+
+	// Server for origin A
+	muxA := http.NewServeMux()
+	muxA.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Set("Content-Type", "text/html; charset=utf-8")
+		_, err := w.Write([]byte(originAHTML))
+		require.NoError(t, err)
+	})
+	muxA.HandleFunc("/innerA", func(w http.ResponseWriter, r *http.Request) {
+		w.Header().Set("Content-Type", "text/html; charset=utf-8")
+		_, err := w.Write([]byte(innerAHTML))
+		require.NoError(t, err)
+	})
+	srvA := httptest.NewServer(muxA)
+	t.Cleanup(func() {
+		srvA.Close()
+	})
+
+	t.Run("ok/click_in_nested_frames", func(t *testing.T) {
+		t.Parallel()
+
+		// Use srvA.URL as the entry point in the rest of the test (navigate, click, etc.).
+		page := newTestBrowser(t).NewPage(nil)
+
+		// Navigate to the page that srvA is serving.
+		opts := &common.FrameGotoOptions{
+			Timeout: common.DefaultTimeout,
+		}
+		_, err := page.Goto(srvA.URL, opts)
+		require.NoError(t, err)
+
+		var (
+			clickOpts     = common.NewFrameClickOptions(page.Timeout())
+			expectedCount = "1"
+		)
+
+		// First click on the main frame.
+		err = page.Locator("#incrementA", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countA, ok, err := page.Locator("#countA", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countA)
+
+		// Now get the first nested frame.
+		frameA, err := page.Query("#frameA")
+		require.NoError(t, err)
+
+		frameAContent, err := frameA.ContentFrame()
+		require.NoError(t, err)
+
+		// Click on the second nested frame.
+		err = frameAContent.Locator("#incrementA2", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countA2, ok, err := frameAContent.Locator("#countA2", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countA2)
+
+		// Now get the second nested frame.
+		frameB, err := page.Query("#frameB")
+		require.NoError(t, err)
+
+		frameBContent, err := frameB.ContentFrame()
+		require.NoError(t, err)
+
+		// Click on the third nested frame.
+		err = frameBContent.Locator("#incrementB", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countB, ok, err := frameBContent.Locator("#countB", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countB)
+
+		// Now get the third nested frame.
+		frameC, err := frameBContent.Query("#frameC", false)
+		require.NoError(t, err)
+
+		frameCContent, err := frameC.ContentFrame()
+		require.NoError(t, err)
+
+		// Click on the fourth nested frame.
+		err = frameCContent.Locator("#increment", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		count, ok, err := frameCContent.Locator("#count", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, count)
+
+		// Now get the fourth nested frame.
+		frameD, err := frameCContent.Query("#frameD", false)
+		require.NoError(t, err)
+
+		frameDContent, err := frameD.ContentFrame()
+		require.NoError(t, err)
+
+		// Click on the fifth nested frame.
+		err = frameDContent.Locator("#incrementD", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countD, ok, err := frameDContent.Locator("#countD", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countD)
+	})
+
+	// This test is the same as the previous one, but uses the Locator API
+	// instead of the Frame APIs.
+	t.Run("ok/click_in_nested_frames_with_locator", func(t *testing.T) {
+		t.Parallel()
+
+		// Use srvA.URL as the entry point in the rest of the test (navigate, click, etc.).
+		page := newTestBrowser(t).NewPage(nil)
+
+		// Navigate to the page that srvA is serving.
+		opts := &common.FrameGotoOptions{
+			Timeout: common.DefaultTimeout,
+		}
+		_, err := page.Goto(srvA.URL, opts)
+		require.NoError(t, err)
+
+		var (
+			clickOpts     = common.NewFrameClickOptions(page.Timeout())
+			expectedCount = "1"
+		)
+
+		// First click on the main frame.
+		err = page.Locator("#incrementA", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countA, ok, err := page.Locator("#countA", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countA)
+
+		// Now get the first nested frame.
+		frameAContent := page.Locator("#frameA", nil).ContentFrame()
+
+		// Click on the second nested frame.
+		err = frameAContent.Locator("#incrementA2", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countA2, ok, err := frameAContent.Locator("#countA2", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countA2)
+
+		// Now get the second nested frame.
+		frameBContent := page.Locator("#frameB", nil).ContentFrame()
+
+		// Click on the third nested frame.
+		err = frameBContent.Locator("#incrementB", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countB, ok, err := frameBContent.Locator("#countB", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countB)
+
+		// Now get the third nested frame.
+		frameCContent := frameBContent.Locator("#frameC", nil).ContentFrame()
+
+		// Click on the fourth nested frame.
+		err = frameCContent.Locator("#increment", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		count, ok, err := frameCContent.Locator("#count", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, count)
+
+		// Now get the fourth nested frame.
+		frameDContent := frameCContent.Locator("#frameD", nil).ContentFrame()
+
+		// Click on the fifth nested frame.
+		err = frameDContent.Locator("#incrementD", nil).Click(clickOpts)
+		require.NoError(t, err)
+
+		countD, ok, err := frameDContent.Locator("#countD", nil).TextContent(nil)
+		require.NoError(t, err)
+		assert.True(t, ok)
+		assert.Equal(t, expectedCount, countD)
+	})
 }
