@@ -697,5 +697,5 @@ func TestExceptionInHandleSummaryFallsBackToTextSummary(t *testing.T) {
 	errMsg, err := logErrors[0].String()
 	require.NoError(t, err)
 	assert.Contains(t, errMsg, "\"Error: intentional error\\n\\tat file:///script.js:4:11(3)\\n")
-	assert.Equal(t, logErrors[0].Data, logrus.Fields{"hint": "script exception"})
+	assert.Equal(t, logrus.Fields{"hint": "script exception"}, logErrors[0].Data)
 }
